<?php

declare(strict_types=1);

namespace App\Controller\Api\v1;

use App\Controller\AbstractController;
use App\Utils\AppResult;
use App\Utils\Control\AuthUser;
use Hyperf\DbConnection\Db;

//
use App\Request\Api\v1\Player\RecordRequest;
use App\Master\Enum\RedisKeyEnum;
use App\Utils\RedisUtil;
use App\Service\QueueService;
use Hyperf\Di\Annotation\Inject;

/**
 * 选手接口
 * PlayerController
 */
class PlayerController extends AbstractController
{
    // 日志模块名称
    const LOG_MODULE = 'v1/PlayerController';

    #[Inject]
    protected QueueService $service;

    //给选手投票
    public function record(RecordRequest $request){
        //验证活动开始时间，结束时间
        if( time() < strtotime(site('subject_starttime')) )
        {
            return AppResult::error('活动开始时间：'.site('subject_starttime'));
        }
        if( strtotime(site('subject_endtime')) < time() )
        {
            return AppResult::error('活动在'.site('subject_endtime').'就结束了');
        }

        if (!RedisUtil::getInstance(RedisKeyEnum::API_REQUEST_TRAFFIC)->tryTimes(1, 3)) {
            return AppResult::error('操作太快了,休息一下吧');
        }

        $params = $request->validated();// 获取校验通过的参数
        $user   = AuthUser::getInstance()->get();

        $player_id = intval($params['player_id']);
        $vote      = intval($params['vote']);
        if(!$player_id || !$vote){
            return AppResult::error('投票失败');
        }

        //登录用户票数的检查
        $user_id = $user['id'];
        //今天,投了几票
        $today_record   = RedisUtil::getInstance(RedisKeyEnum::VOTE_RECORD.date('Y-m-d').':'.$user_id)->get();
        //今天，用户答对的次数
        $question_vote  = RedisUtil::getInstance(RedisKeyEnum::EAXM_RIGHT.date('Y-m-d').':'.$user_id)->get();
        $question_vote  = intval(bcmul((string)$question_vote,(string)site('question_right_getvotenum'),0));//乘以基数，也就是答题获得的票数
        //今天，用户答题的次数
        $today_question = RedisUtil::getInstance(RedisKeyEnum::EAXM_TIMES.date('Y-m-d').':'.$user_id)->get();
        //今天,免费的的票
        $gift_votes = site('gift_votes_user_eday');
        //今天,免费的答题次数
        $exam_times = site('exam_times_user_eday');

        //免费票 + 答对票数 - 已投票次数 < vote
        $times = $gift_votes + $question_vote - $today_record;
        if($times < 0){$times = 0;}

        //票数不够
        if($times < $vote){
            $msg = '投票失败,您今日剩余投票次数: '.$times.'次';

            //剩余投票次数，剩余答题次数
            $result = [
                'uservote' => $times,
                'userquestion' => $exam_times - $today_question,
            ];

            return AppResult::error($msg,$result);
        }

        //检查选手
        $player_info = Db::table('vote_player')->select(['id','subject_id','votes'])->where('id',$player_id)->first();
        if(!$player_info){
            return AppResult::error('不存在的选手');
        }

        //今日投票次数，自增一次
        $today_record = RedisUtil::getInstance(RedisKeyEnum::VOTE_RECORD.date('Y-m-d').':'.$user['id'])->incrby_expire($vote,86400);

        //给选手加票
        $this->service->playerPush(['player_id' => $player_id, 'user_id' => $user['id'], 'vote' => $vote], 0);
        //日志


        //免费票 + 答对次数 - 已投票次数
        $times = $gift_votes + $question_vote - $today_record;  //重新计算一次，是因为$today_record 自增了
        if($times < 0){$times = 0;}

        //
        $msg = '投票成功,您今日剩余投票次数: '.$times.'次';

        //剩余投票次数，剩余答题次数
        $result = [
            'uservote' => $times,
            'userquestion' => $exam_times - $today_question,
        ];

        return AppResult::success($msg,$result);
    }




}