<?php

declare(strict_types=1);

namespace App\Job;

use App\Utils\LogUtil;
use Hyperf\AsyncQueue\Job;
use Hyperf\DbConnection\Db;

class QuestionJob extends Job
{
    //日志板块
private const LOG_MODULE = 'QuestionJob';

    public $params;

    /**
     * 任务执行失败后的重试次数，即最大执行次数为 $maxAttempts+1 次
     */
    protected int $maxAttempts = 2;

    public function __construct($params)
    {
        // 这里最好是普通数据，不要使用携带 IO 的对象，比如 PDO 对象
        $this->params = $params;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        //日志统一写入
        LogUtil::getInstance('Queues/');//设置日志存入通道
        LogUtil::info('开始处理', self::LOG_MODULE, __FUNCTION__, ['params' => $this->params]);
        // 根据参数处理具体逻辑
        // 通过具体参数获取模型等
        // 这里的逻辑会在 ConsumerProcess 进程中执行
//        var_dump($this->params);

        $res = $this->submit($this->params['log_data'],$this->params['is_right_number'],$this->params['bind_jigou_id']);

        LogUtil::info('处理结果', self::LOG_MODULE, __FUNCTION__,$res);
        LogUtil::close();
    }

    public function submit($logdata,$is_right_number,$bind_jigou_id){
        Db::beginTransaction();

        //答题日志
        $log_id = Db::table('user_question_log')->insert($logdata);
        if(!$log_id){
            Db::rollBack();
            return false;
        }

        if($is_right_number > 0){
            //给机构加分
            $rs = Db::table('vote_jigou')->where('id',$bind_jigou_id)->increment('score',$is_right_number);
            if(!$rs){
                Db::rollBack();
                return false;
            }
        }

        Db::commit();
        return true;

    }
}