<?php

namespace App\Master\Framework\Library\Tencent;

use App\Master\Framework\Library\Library;
use function Hyperf\Config\config;

class TencentIm extends Library
{
    public string $base_uri = "https://console.tim.qq.com";
    private mixed $config;

    /**
     * 实例化
     */
    public function __construct()
    {
        // 获取配置信息
        $this->config = [
            'appid'      => site('tencent_im_appid'),
            'identifier' => site('tencent_im_identifier'),
            'key'        => site('tencent_im_key'),
        ];
    }

    /**
     * 获取直播群在线人数
     * @param int $room_no
     * @param array $counter
     * @return bool
     */
    public function get_online_member_num(int $room_no)
    {
        $data     = [
            'GroupId' => (string)$room_no
        ];
        $response = $this->post('/v4/group_open_http_svc/get_online_member_num', $data);
        if ($response->getStatusCode() != 200) {
            return $this->error($response->getReasonPhrase());
        }

        $json = $response->getBody()->getContents();
        $body = json_decode($json, true);
        if (empty($body['ActionStatus']) || $body['ActionStatus'] != 'OK') {
            return $this->error(!empty($body['ErrorInfo']) ? $body['ErrorInfo'] : 'im error', $body ?? []);
        }

        return $this->success('获取成功', $body);
    }

    /**
     * 设置计数器
     * @param int $room_no
     * @param array $counter
     * @return bool
     */
    public function update_group_counter(int $room_no, array $counter)
    {
        $data     = [
            'GroupId'      => (string)$room_no,// 群组 ID
            'GroupCounter' => $counter,
            'Mode'         => 'Set'
        ];
        $response = $this->post('/v4/group_open_http_svc/update_group_counter', $data);
        if ($response->getStatusCode() != 200) {
            return $this->error($response->getReasonPhrase());
        }

        $json = $response->getBody()->getContents();
        $body = json_decode($json, true);
        if (empty($body['ActionStatus']) || $body['ActionStatus'] != 'OK') {
            return $this->error(!empty($body['ErrorInfo']) ? $body['ErrorInfo'] : 'im error', $body ?? []);
        }

        return $this->success('获取成功', $body);
    }

    /**
     * 设置群属性
     * @param int $room_no
     * @param array $attr
     * @return bool
     */
    public function modify_group_attr(int $room_no, array $attr)
    {
        $data     = [
            'GroupId'   => (string)$room_no,// 群组 ID
            'GroupAttr' => $attr
        ];
        $response = $this->post('/v4/group_open_http_svc/modify_group_attr', $data);
        if ($response->getStatusCode() != 200) {
            return $this->error($response->getReasonPhrase());
        }

        $json = $response->getBody()->getContents();
        $body = json_decode($json, true);
        if (empty($body['ActionStatus']) || $body['ActionStatus'] != 'OK') {
            return $this->error(!empty($body['ErrorInfo']) ? $body['ErrorInfo'] : 'im error', $body ?? []);
        }

        return $this->success('获取成功', $body);
    }

    /**
     * 获取全部群组
     * @return bool
     */
    public function get_appid_group_list()
    {
        $data     = [
            'Limit' => 1000,// 群主 ID
            'Next'  => 0
        ];
        $response = $this->post('/v4/group_open_http_svc/get_appid_group_list', $data);
        if ($response->getStatusCode() != 200) {
            return $this->error($response->getReasonPhrase());
        }

        $json = $response->getBody()->getContents();
        $body = json_decode($json, true);
        if (empty($body['ActionStatus']) || $body['ActionStatus'] != 'OK') {
            return $this->error(!empty($body['ErrorInfo']) ? $body['ErrorInfo'] : 'im error', $body ?? []);
        }

        return $this->success('获取成功', $body);
    }

    /**
     * 获取全部群组
     * @return bool
     */
    public function delete_chat(string $user_id,string $to_user_id)
    {
        $data     = [
            'From_Account' => $user_id,// 用户 ID
            'Type'  => 1,
            'To_Account'  => $to_user_id,
            'ClearRamble'  => 1,
        ];
        $response = $this->post('/v4/recentcontact/delete', $data);
        if ($response->getStatusCode() != 200) {
            return $this->error($response->getReasonPhrase());
        }

        $json = $response->getBody()->getContents();
        $body = json_decode($json, true);
        if (empty($body['ActionStatus']) || $body['ActionStatus'] != 'OK') {
            return $this->error(!empty($body['ErrorInfo']) ? $body['ErrorInfo'] : 'im error', $body ?? []);
        }

        return $this->success('获取成功', $body);
    }

    private function post(string $uri, array $params = [])
    {
        $random  = rand(10000000, 99999999);
        $userSig = $this->usersig($this->config['identifier']);
        return $this->postJson("{$uri}?sdkappid={$this->config['appid']}&identifier={$this->config['identifier']}&usersig={$userSig}&random={$random}&contenttype=json", $params);
    }

    /**
     * 获取usersig签名-具体操作
     */
    public function userSig($user_id)
    {
        // 获取配置信息
        $userSigObj = new GetUserSig($this->config["appid"], $this->config["key"]);
        return $userSigObj->genUserSig($user_id);
    }
}