<?php

declare(strict_types=1);

namespace App\Model\Arts;

use App\Model\Model;

class DriverModel extends Model
{
    /**
     * The table associated with the model.
     *
     * @var ?string
     */
    protected ?string $table = 'driver';

    protected ?string $dateFormat = 'U';
    public bool       $timestamps = false;

    protected int $is_status_search = 1;// 默认使用 status = 1 筛选
    protected int $is_delete_search = 0;// 默认使用 is_delete = 0 筛选

    /**
     * 默认查询字段
     *
     * @var array|string[]
     */
    public array $select = [
        '*'
    ];

    /**
     * 中间件中获取用户信息
     * @param int $user_id
     * @return array
     */
    public function authUserInfo(int $user_id)
    {
        return $this->getDetail(params: ['id' => $user_id]);
    }

    /**
     * 校验账户
     * @param string $mobile
     * @return array
     */
    public static function checkExists(string $mobile = '', string $password = '', string $area_code = ''): array
    {
        $query = self::query();

        $query->where('mobile', $mobile);
        !empty($area_code) && $query->where('area_code', $area_code);

        $query->whereIn('status', [0, 1]);

        $exists = $query->exists();
        $driver = $query->first();

        $driver_id    = $driver['id'] ?? 0;
        $old_password = $driver['password'] ?? '';
        if (!empty($password) && !password_verify(md5($password), $old_password)) {
            $exists = false;
        }

        return [$exists, $driver_id];
    }

    /**
     * 创建账号
     * @param array $params
     * @return bool|int
     */
    public static function createAccount(array $params): bool|int
    {
        if (empty($params['mobile'])) {
            return false;
        }

        // 车辆信息
        $car_id = $params['car_id'] ?? 0;
        $car_params = [
            'car_img'        => $params['car_img'] ?? '',
            'car_color'        => $params['car_color'] ?? '',
        ];

        // 驾照信息
        $license_id = $params['license_id'] ?? 0;
        $license_params = [
            'license_img' => $params['driver_license_img'] ?? ''
        ];

        unset(
            $params['captcha'],
            $params['car_id'],
            $params['license_id'],
            $params['car_img'],
            $params['car_color'],
            $params['driver_license_img']
        );

        if (!empty($params['password'])) {
            $params['password'] = password_hash(md5($params['password']), PASSWORD_DEFAULT);
        }

        $params['status']      = 1;
        $params['create_time'] = time();

        // 创建账户
        $driver_id = self::query()->insertGetId($params);
        if (!$driver_id) {
            return false;
        }

        // 创建钱包
        if (!DriverWalletModel::add($driver_id)) {
            return false;
        }

        if ($params['type'] == 2){
            // 创建车辆信息
            if (!DriverCarModel::addEdit($driver_id, $car_params)) {
                return false;
            }

            // 创建驾照
            if (!DriverLicenseModel::addEdit($driver_id, $license_params)) {
                return false;
            }
        } else {
            $car = DriverCarModel::query()->where('id',$car_id)->where('driver_id',0)->update(['driver_id' => $driver_id,'update_time'=>time()]);
            $license = DriverLicenseModel::query()->where('id',$license_id)->where('driver_id',0)->update(['driver_id' => $driver_id,'update_time'=>time()]);
            if (!$car || !$license) {
                return false;
            }
        }

        return $driver_id;
    }

    /**
     * 更新登录时间
     * @param int $driver_id
     * @param string $openid
     * @param string $unionid
     * @param array $extend
     * @return bool
     */
    public static function loginTime(int $driver_id, string $openid = '', string $unionid = '', array $extend = []): bool
    {
        !empty($unionid) && $data['unionid'] = $unionid;
        !empty($openid) && $data['openid'] = $openid;
        $data['last_login_time'] = time();
        $data['push_cid'] = '';

        $update = self::query()->where('id', $driver_id)->update($data);

        return (bool)$update;
    }

    public function edit(int $driver_id, array $extend = [])
    {
        if (!empty($extend['password'])) {
            $extend['password'] = password_hash(md5($extend['password']), PASSWORD_DEFAULT);
        }
        $extend['update_time'] = time();
        if (!$this->query()->where('id', $driver_id)->update($extend)){
            return $this->error('更新失败');
        }
        return $this->success('更新成功');
    }

    public function car()
    {
        return $this->hasOne(DriverCarModel::class, 'driver_id', 'id');
    }
}
