<?php

declare(strict_types=1);

namespace App\Model\Arts;

use App\Model\Model;
use Hyperf\DbConnection\Db;

class LiveRoomPkModel extends Model
{
    /**
     * The table associated with the model.
     *
     * @var ?string
     */
    protected ?string $table = 'live_room_pk';

    protected ?string $dateFormat = 'U';

    public bool $timestamps = false;

    protected int $is_status_search = 1;// 默认使用 status = 1 筛选
    protected int $is_delete_search = 0;// 默认使用 is_delete = 0 筛选

    /**
     * 默认查询字段
     *
     * @var array|string[]
     */
    public array $select = [
        '*'
    ];

    public function searchRoomNoAttribute($query, $value, array $params): mixed
    {
        if (empty($value)) {
            return $query;
        }
        return $query->where('room_no', $value);
    }

    public function searchSessionAttribute($query, $value, array $params): mixed
    {
        if (empty($value)) {
            return $query;
        }
        return $query->where('session', $value);
    }

    /**
     * 结束PK
     * @param int $pk_id
     * @return bool
     */
    public function endPk(int $pk_id): bool
    {
        $pk_info = $this->query()->where('id',$pk_id)->first();
        if (!$pk_info){
            return $this->error('pk不存在');
        }
        if ($pk_info['status'] == 0) {
            return $this->success('已结束');
        }
        Db::beginTransaction();
        try {
            $time = time();
            if (!$this->query()->where('id', $pk_info['id'])->update(['status' => 0, 'end_time' => $time, 'duration' => Db::raw("{$time} - start_time")])) {
                Db::rollback();
                return $this->error('结束失败');
            }

            $redRooms  = LiveRoomModel::query()->where('id', $pk_info['red_room_id'])->where('pk_id', $pk_info['id'])->update(['pk_id' => 0, 'is_pk' => 0, 'pk_type' => 0]);
            $blueRooms = LiveRoomModel::query()->where('id', $pk_info['blue_room_id'])->where('pk_id', $pk_info['id'])->update(['pk_id' => 0, 'is_pk' => 0, 'pk_type' => 0]);
            if (!$redRooms || !$blueRooms) {
                Db::rollback();
                return $this->error('操作失败');
            }
            Db::commit();
            return $this->success('已结束');
        } catch (\Throwable $e) {
            Db::rollback();
            return $this->error($e->getMessage());
        }
    }
}
