<?php

declare(strict_types=1);

namespace App\Request\Api\v1\Order;

use Hyperf\Validation\Request\FormRequest;

class PageRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     */
    public function authorize(): bool
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     */
    public function rules(): array
    {
        return [
            'type'             => 'required|integer',
            'point'            => 'required|array',
            'people_num'       => 'nullable|integer',
            'luggage_num'      => 'nullable|integer',
            'is_back'          => 'nullable|integer',
            'is_exclusive'     => 'nullable|integer',
            'appointment_time' => 'nullable|string',
            'coupon_id'        => 'nullable|integer',
        ];
    }

    /**
     * 表单请求后钩子
     * @param $validator
     */
    public function withValidator($validator)
    {
        $validator->after(function ($validator) {
            //获取参数
            $params = $this->validationData();

            if (!isset($params['point']) || (count($params['point']) < 2 || count($params['point']) > 3)) {
                $validator->errors()->add('point', '起始点数据有误');
            }

            if (isset($params['point'])) {
                foreach ($params['point'] as $item) {
                    if (empty($item['address']) || empty($item['lng']) || empty($item['lat'])) {
                        $validator->errors()->add('point', '起始点数据有误');
                    }

                    if (!empty($params['type']) && in_array($params['type'], [10, 20]) && empty($item['postal_code'])) {
                        $validator->errors()->add('point', '起始点数据有误');
                    }
                }
            }

        });
    }
}
